/* -----------------------------------------------------------
                         test_max7219.c

   Demoprogramm zur 8x8 Leuchtdiodenmatrix mit MAX7219
   (Chinamodul)


   Benoetigte Hardware;

            - 8x8 LED Matrix mit MAX7219 IC

   MCU   :  STM8S103F3
   Takt  :  externer Takt 16 MHz


   29.08.17 by R. Seelig

   Pins und Anschlussbelegung siehe max7219.h
   ----------------------------------------------------------- */

#include "stm8s.h"
#include "stm8_init.h"
#include "stm8_gpio.h"

#include "max7219.h"
#include "font8x8_vLSB.h"


// "Framebuffer" der ein komplettes 8x8 Pixelbitmap aufnehmen kann
uint8_t fbuf[8] =
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };


/* -----------------------------------------------------------
     fbuf_putpixel

     setzt in einem 8 Byte grossen Pufferspeicher ein
     einzelnes Bit (Pixel)

     Parameter

        *bmp  : Zeiger auf einen 8 Byte grossen Pufferspeicher
        x     : X-Koordinate des zu setzenden Pixels
        y     : Y-Koordinate des zu setzenden Pixels
        c     : "Farbe" = 1 : LED leuchtet
                        = 0 : LED aus
   -----------------------------------------------------------*/
void fbuf_putpixel(uint8_t *bmp, uint8_t x, uint8_t y, uint8_t c)
{
  if (c)
    bmp[x] |= (1 << y);
  else
    bmp[x] &= ~(1 << y);
}

/* -----------------------------------------------------------
     fbuf_clr

     loescht einen 8 Byte grossen Pufferspeicher in dem
     alle 64 Bits geloescht werden
   ----------------------------------------------------------- */
void fbuf_clr(uint8_t *bmp)
{
  uint8_t i;

  for (i= 0; i< 8; i++) bmp[i]= 0;
}

/* -----------------------------------------------------------
     fbuf_shl_ins

     verschiebt alle Spalten des Zielspeichers *dest um eine
     Stelle nach links und fuegt am eine eine Spaltee aus
     *src ein
   ----------------------------------------------------------- */
void fbuf_shl_ins(uint8_t *dest, uint8_t *src)
{
  uint8_t i;

  for (i= 0; i< 7; i++) dest[i]= dest[i+1];
  dest[7]= *src;
}

/* -----------------------------------------------------------
     fbuf_scroll_in

     "Scrollt" ein 8x8 grosses Image von links in die LED-
     matrix (die in *fbuf enthalten ist) ein.

       *src  : einzuscrollendes Image
       dtime : Scrollgeschwindigkeit in mS
   ----------------------------------------------------------- */

void fbuf_scroll_in(uint8_t *src, int dtime)
{
  uint8_t cx;

  for (cx= 0; cx< 8; cx++)
  {
    fbuf_shl_ins(&fbuf[0], &src[cx]);
    m7219_setbmp(&fbuf[0]);
    delay(dtime);
  }
}

/* -----------------------------------------------------------
     fbuf_scroll_text_in

     Scrollt einen Text in die Matrix ein und wartet nach
     jedem eingescrollten Buchstaben eine Zeitdauer, bevor
     der naechste Buchstabe eingeschoben wird

       *txt   : Zeiger auf den Text, der auf der LED-Matrix
                angezeigt werden soll
       dtime1 : Zeidauer in mS zwischen den Scrollschritten
       dtime2 : Zeitdauer in mS die ein Buchstabe auf der
                LED-Matrix angezeigt wird, bevor der naechste
                Buchstabe eingescrollt wird.
   ----------------------------------------------------------- */
void fbuf_scroll_text_in(uint8_t *txt, int dtime1, int dtime2)
{
  uint8_t c;

  do
  {
    c = *txt;
    if (c)
    {
      fbuf_scroll_in(&font8x8[c][0], dtime1);
      delay(dtime2);
    }
  }while ( *txt++ );

}

/* -----------------------------------------------------------
                             M-A-I-N
   ----------------------------------------------------------- */
int main(void)
{
  uint8_t cx;
  uint8_t x,y;

  sysclock_init(0);                     // Initialisieren mit internem RC (16 MHz)

  m7219_init();                         // Anschluesse des STM8 die die Matrix steuern
                                        // als Ausgang setzen und die Register des
                                        // MAX7219 konfigurieren

  while(1)                              // endlose Demowiederholung
  {
    fbuf_clr(&fbuf[0]);                 // Anzeigebuffer loeschen

    // "rotierendes" Lauflicht
    for (cx= 0; cx < 3; cx++)
    {
      // obere und untere Reihe gegenlaeufiges  Lauflicht
      for (x= 0; x< 8; x++)
      {
        // Pixel im Framebuffer setzen
        fbuf_putpixel(&fbuf[0], x, 0, 1);
        fbuf_putpixel(&fbuf[0], 7-x, 7, 1);
        // und diesen anzeigen
        m7219_setbmp(&fbuf[0]);

        delay(70);

        // Pixel im Framebuffer setzen
        fbuf_putpixel(&fbuf[0], x, 0, 0);
        fbuf_putpixel(&fbuf[0], 7-x, 7, 0);
        // und diesen anzeigen
        m7219_setbmp(&fbuf[0]);
      }

      // vordere und hintere Spalte gegenlaeufiges Lauflicht
      for (y= 1; y< 7; y++)
      {
        // Pixel im Framebuffer setzen
        fbuf_putpixel(&fbuf[0], 7, y, 1);
        fbuf_putpixel(&fbuf[0], 0, 7-y, 1);
        // und diesen anzeigen
        m7219_setbmp(&fbuf[0]);

        delay(70);

        // Pixel im Framebuffer setzen
        fbuf_putpixel(&fbuf[0], 7, y, 0);
        fbuf_putpixel(&fbuf[0], 0, 7-y, 0);
        // und diesen anzeigen
        m7219_setbmp(&fbuf[0]);
      }
    }
    // Ausgangsposition des Lauflichts anzeigen (als letzten Schritt des Lauflichts)
    fbuf_clr(&fbuf[0]);
    fbuf_putpixel(&fbuf[0], 0, 0, 1);
    fbuf_putpixel(&fbuf[0], 7, 7, 1);
    m7219_setbmp(&fbuf[0]);
    delay(1000);

    fbuf_clr(&fbuf[0]);
    m7219_clrscr();

    // einen Text auf der Matrix scrollen
    fbuf_scroll_text_in("STM8", 100, 1000);
    delay(1000);

    // "Herzchen" (Ascii-Code 3) blinken lassen
    for (cx= 0; cx< 6; cx++)
    {
      m7219_setbmp(&font8x8[3][0]);
      delay(200);
      m7219_clrscr();
      delay(200);
    }
  }
}
